//
//  Biquad.cpp
//
//  Created by Nigel Redmon on 11/24/12
//  EarLevel Engineering: earlevel.com
//  Copyright 2012 Nigel Redmon
//
//  For a complete explanation of the Biquad code:
//  http://www.earlevel.com/main/2012/11/26/biquad-c-source-code/
//
//  License:
//
//  This source code is provided as is, without warranty.
//  You may copy and distribute verbatim copies of this document.
//  You may modify and use this source code to create binary code
//  for your own purposes, free or commercial.
//
//  Modified by Will Saba and Nick Barany 6/8/2015, all filter formulas
//  replaced with SigmaStudio equivalents (type 2 filters)
//  See: http://wiki.analog.com/resources/tools-software/sigmastudio/toolbox/filters/general2ndorder
//  or,  https://ez.analog.com/thread/42007
//  for details


#include <math.h>
#include "Biquad.h"


Biquad::Biquad() {
  //Default instantiated filter parameters, since peakGain = 0, default filter does nothing
    type = bq_type_peak;
    a0 = 1.0;
    b0 = a1 = a2 = b1 = b2 = 0.0;
    Fc = 0.001042; // ~100 Hz
    Q = 0.707;
    peakGain = 0.0;
    z1 = z2 = 0.0;
}

Biquad::Biquad(int type, double Fc, double Q, double peakGainDB) {
    setBiquad(type, Fc, Q, peakGainDB);
    z1 = z2 = 0.0;
}

Biquad::~Biquad() {
}

void Biquad::setType(int type) {
    this->type = type;
    calcBiquad();
}

void Biquad::setQ(double Q) {
    this->Q = Q;
    calcBiquad();
}

void Biquad::setFc(double Fc) {
    this->Fc = Fc;
    calcBiquad();
}

void Biquad::setPeakGain(double peakGainDB) {
    this->peakGain = peakGainDB;
    calcBiquad();
}
//Note, Fc is passed as normalized frequency
void Biquad::setBiquad(int type, double Fc, double Q, double peakGainDB) {
    this->type = type;
    this->Q = Q;
    this->Fc = Fc;
    setPeakGain(peakGainDB);
}

void Biquad::calcBiquad(void) {
    
    double A = pow(10, peakGain / 40.0); // 
    
    double alpha;
    double gainLinear = 1;//pow(10, peakGain/20);  1 = pass through rest of spectrum, only boost fc
    double w0;
    w0 = 2*M_PI*Fc;
    double dw0,b;
    double S = Q; // Slope parameter passed as Q, keep range 0->2

// Used in Notch filter, notation comes from Analog Devices thread, see top of file
    dw0 = w0/Q;
    b = 1 / (1 + tan(dw0 / 2));

    switch (this->type) {
        case bq_type_lowpass://BW
             alpha = sin(w0)/2 * 1/sqrt(2);
             a0 = 1+alpha;
             a1 = -2*cos(w0);
             a2 = 1-alpha;
             b0 = (1-cos(w0))*gainLinear/2;
             b1 = 1-cos(w0)*gainLinear;
             b2 = (1-cos(w0))*gainLinear/2;
            break;
        case bq_type_lowpassLR://Same as "lowpass" except alpha, 2nd order stage
             alpha = sin(w0)/(2*(1/sqrt(2)));
             a0 = 1+alpha;
             a1 = -(2*cos(w0));
             a2 = 1-alpha;             
             b1 = (1-cos(w0))*gainLinear;
             b0 = b1/2;
             b2 = b0;
            break;
            
        case bq_type_highpass:
             alpha = sin(w0)/2 * 1/sqrt(2);
             a0 = 1+alpha;
             b0 = (1+cos(w0))*gainLinear/2;
             b1 = -(1+cos(w0))*gainLinear;
             a1 = -2*cos(w0);
             b2 = (1+cos(w0))*gainLinear/2;
             a2 = 1-alpha;          
            break;
        case bq_type_highpassLR: // Same as highpass except alpha, 2nd order stage
             alpha = sin(w0)/(2*(1/sqrt(2)));
             a0 = 1+alpha;
             a1 = -2*cos(w0);
             a2 = 1-alpha;   
             b1 = -(1+cos(w0))*gainLinear;
             b0 = -b1/2;
             b2 = b0;            
            break;
            
        case bq_type_bandpass: // Not implemented, not useful for this application
            
            break;
//Notch filter with variable width based on passed "Q" parameter... 
//   altering the "S" or slope parameter defined in Sigma Studio            
        case bq_type_notch:
            a0 = 1;
            b0 = gainLinear*b;
            b1 = gainLinear*(-2*b*cos(w0));
            b2 = gainLinear*b;
            a1 = -2*b*cos(w0);
            a2 = (2*b-1);
         
            break;
// Parametric Filter Type            
        case bq_type_peak: 
           alpha = sin(w0)/(2.0000*A*Q);
           a0 = 1.0000+alpha/A;
           a1 = -2.0000*cos(w0);
           a2 = 1.000-alpha/A;
           b0 = (1.0000+alpha*A)*gainLinear;
           b1 = -(2.0000*cos(w0))*gainLinear;
           b2 = (1.0000-alpha*A)*gainLinear;
           /* If you wish to implement an actual "peak" filter (inverse of notch)
           
           a0 = 1;
           a1 = - 2 * b * cos(w0);
           a2 = (2 * b - 1);
           b0 = gainLinear * (1 - b);
           b1 = 0;
           b2 = gainLinear * (b - 1);
            break;
            */
//Primer for shelving filters:
//Variable Slope -> S, passed as Q, '1' gives a smooth transition,
//  greater slope increases transition rate, and yields an overdamped looking frequency response
//  lower slope decreases and yields underdamped looking frequency response.
        case bq_type_lowshelf:
            alpha = sin(w0)/2.0000000*sqrt(((double)A+1.000000/A)*(1.0000/S-1)+2);
            a0= (A+1) + (A-1)*cos(w0) + 2*sqrt(A)*alpha;
            a1= -2*((A-1)+(A+1)*cos(w0));
            a2 = (A+1) + (A-1)*cos(w0) - 2*sqrt(A)*alpha;
            b0 = A*((A+1)-(A-1)*cos(w0)+2*sqrt(A)*alpha)*gainLinear;
            b1 = 2*A*((A-1)-(A+1)*cos(w0))*gainLinear;
            b2 = A*((A+1) - (A-1)*cos(w0)-2*sqrt(A)*alpha)*gainLinear;
            break;
        case bq_type_highshelf:
            
            alpha = sin(w0)/2.000000*sqrt((A+1.000000/A)*(1.00000/S-1)+2);
            a0= (A+1) - (A-1)*cos(w0) + 2*sqrt(A)*alpha;
            a1= 2*((A-1)-(A+1)*cos(w0));
            a2 = (A+1) - (A-1)*cos(w0) - 2*sqrt(A)*alpha;
            b0 = A*((A+1)+(A-1)*cos(w0)+2*sqrt(A)*alpha)*gainLinear;
            b1 = -2*A*((A-1)+(A+1)*cos(w0))*gainLinear;
            b2 = A*((A+1) + (A-1)*cos(w0)-2*sqrt(A)*alpha)*gainLinear;
            break;
    }//End of filter types
    //Normalize all values by dividing by a0 to make a0 1 (not stored in dsp)
   // -> thus only requiring 5 parameters to define each filter
    a1 = (double)a1/(double)a0;
    a2 = (double)a2/(double)a0;
    b0 = (double)b0/(double)a0;
    b1 = (double)b1/(double)a0;
    b2 = (double)b2/(double)a0;
   //invert A1, A2 for parameter ram storage, see Wiki for explanation
    a1 = a1*-1;
    a2 = a2*-1;
    
    return;
}

