/*
 * File:			SigmaStudioFW.h
 *
 * Description:  	SigmaStudio System framework macro and function declarations
 *				
 * This file/software originated as an Analog Devices copyrighted product but since has been
 * completely rewritten and stripped of copyrighted content for the express purposes
 * of integrating the arduino uno with the ADAU 1702 EVAL board and helping others
 * trying to do the same. As such: 
//  This source code is provided as is, without warranty.
//  You may copy and distribute verbatim copies of this document.
//  You may modify and use this source code to create binary code
//  for your own purposes, free or commercial.
 * Purpose:
 *         This document contains various methods of computing and sending parameter data over i2c to the
 *         ADAU 1702 dev board. Individual function comments explain there purposes and in most cases
 *         multiple functions have been implemented where fewer is possible for the sake of readability
 * Last Revised: 6/9/2015 by Will Saba and Nick Barany
 * If there are improvements that can/should be made feel free to contact me, Will Saba, at bartallen101@gmail.com
 */	


 /*******************--#includes -> reference libraries and register address definitions--******************/
#ifndef __SIGMASTUDIOFW_H__
#define __SIGMASTUDIOFW_H__

#include "Biquad.h"
/**  Stores location of all default parameters and their respective locations in DSP memory */
#include "main_final_IC_1_PARAM.h"
/**  Stores location of all default register settings and their respective locations in DSP memory */
#include "main_final_IC_1_REG.h"

/*********************  Safeload registers used to load parameters on the fly without audible glitching ***/
#define SAFELOAD_ADDR1 0x0815
#define SAFELOAD_ADDR2 0x0816
#define SAFELOAD_ADDR3 0x0817
#define SAFELOAD_ADDR4 0x0818
#define SAFELOAD_ADDR5 0x0819
#define SAFELOAD_DATA1 0x0810
#define SAFELOAD_DATA2 0x0811
#define SAFELOAD_DATA3 0x0812
#define SAFELOAD_DATA4 0x0813
#define SAFELOAD_DATA5 0x0814
// Define double versions of 1 and 0 and 1/sqrt(2) to reduce instances of magic numbers
#define UNO 1.0
#define ZERO 0.0
#define HALF_POWER 0.707945823669434
// Sub mode return value if found "SUBMODE" as first word on its own line in filterImport
#define SUB_MODE_FLAG_HIGH 2
/**  Numeric constant required by system volume control to set the slew rate */
#define VOLUME_STEP 0.000244140625
/**  Sample Rate of DSP*/
#define sampleRate 96000.0

/**  Experimentally determined I2C address(es) for all inter-board communication */
#define I2C_ADDR_ADAU1702W 0x34 // i2c Write address experimental of ADAU DSP board
#define I2C_ADDR_ADAU1702R 0x50 // i2c Read address experimental of ADAU DSP board
/** Byte length of standard DSP registers = 2 (such as core control) */
#define Address_Length 2
//Unused typedef
//typedef unsigned short ADI_DATA_U16;

/** Define PROGMEM type to store byte constant DSP code in program memory and extract on-demand to overcome memory limitations */
typedef PROGMEM const unsigned char ADI_REG_TYPE;

/*************************--Functions that write to DSP--************************************/
//Initialiize safeload transfer by toggling respective IST bit in control register//
void set_core_IST_bit(int deviceaddress);
//Writes one register of paremter data passed as a double (function converts to fixed point)//
void safeWriteReg(double coeff,short deviceaddress, short data_reg, short addr_reg, short address);
//Writes passed filter Biquad to the DSP with a sequence of 5 safeWriteReg commands and the necesary IST call//
void safeLoadFilter( Biquad* filterX, int addressStart);
//Writes arbitrarily long number of bytes to DSP memory directly//
void SIGMA_WRITE_REGISTER_BLOCK(int devAddress, int address, int length, ADI_REG_TYPE* pData);
//Writes Parameter Memory in 4 byte chunks, length_reg indicates number of registers (not bytes)//
void SIGMA_WRITE_REGISTER_PARAM(int devAddress, int address, int length_reg, ADI_REG_TYPE* pData);
//Writes Program Memory in 5 byte chunks to DSP memory directly, length_reg indicates number of registers (bytes/5)//
void SIGMA_WRITE_REGISTER_PROG(int devAddress, int address, int length_reg, ADI_REG_TYPE* pData);
//Load volume passed as a decimal less than 1, (double) range 0-> 1.00
void setVolume(double coeff, int deviceaddress);
//Load digital fc (need to divide by sampleRate before passing) into 8 biquad filters representing 4th order LR filter//
void setCrossover(double fc, Biquad* LP1L,Biquad* LP1R,Biquad* LP2L,Biquad* LP2R,Biquad* HP1L,Biquad* HP1R,Biquad* HP2L,Biquad* HP2R);
// Parse a txt file and set filters 1-6 for loading, 7-10 to be implemented with increased uC memory
int importFilter(Biquad* filter1,Biquad* filter2,Biquad* filter3,Biquad* filter4,Biquad* filter5,Biquad* filter6,Biquad* filter7,Biquad* filter8,Biquad* filter9, Biquad* filter10);
// Pass address of each parameter location, loads these 3 registers using safeload transfer//
void subModeEnable(int subModeToggleAdd, int rightMuteAdd, int subModeGainAdd);

/* 
 * Parameter data format
 */
#define SIGMASTUDIOTYPE_FIXPOINT 	0
#define SIGMASTUDIOTYPE_INTEGER 	1

/* 
 * Write to a single Device register
 */
#define SIGMA_WRITE_REGISTER( devAddress, address, dataLength, data ) {/*TODO: implement macro or define as function*/}

/* 
 * TODO: CUSTOM MACRO IMPLEMENTATION
 * Write to multiple Device registers 
 */
void setVolume(double coeff, int deviceaddress){
   safeWriteReg( coeff, deviceaddress, SAFELOAD_DATA1, SAFELOAD_ADDR1, MOD_SWVOL1_ALG0_TARGET_ADDR);     // address 1 = 0x01 = target (volume)
   safeWriteReg( VOLUME_STEP , deviceaddress, SAFELOAD_DATA2, SAFELOAD_ADDR2, MOD_SWVOL1_ALG0_STEP_ADDR);// address 2 = 0x02 = step (correlates to slew rate, unchanged)
   set_core_IST_bit(I2C_ADDR_ADAU1702W);
}


void setCrossover(double fc, Biquad* LP1L,Biquad* LP1R,Biquad* LP2L,Biquad* LP2R,Biquad* HP1L,Biquad* HP1R,Biquad* HP2L,Biquad* HP2R){

                LP1L ->setFc(fc);
                LP2L ->setFc(fc);
                LP1R ->setFc(fc);
                LP2R ->setFc(fc);
                HP1L ->setFc(fc);
                HP2L ->setFc(fc);
                HP1R ->setFc(fc);
                HP2R ->setFc(fc);

              //LP1L - alg0_low_filt1
                safeWriteReg (LP1L->b0, I2C_ADDR_ADAU1702W, SAFELOAD_DATA1,SAFELOAD_ADDR1, MOD_CROSSOVER_ALG0_LOW_FILT1_PARAMB0_ADDR); 
		safeWriteReg (LP1L->b1, I2C_ADDR_ADAU1702W, SAFELOAD_DATA2,SAFELOAD_ADDR2, MOD_CROSSOVER_ALG0_LOW_FILT1_PARAMB1_ADDR); 
		safeWriteReg (LP1L->a1, I2C_ADDR_ADAU1702W, SAFELOAD_DATA3,SAFELOAD_ADDR3, MOD_CROSSOVER_ALG0_LOW_FILT1_PARAMA1_ADDR); 
		safeWriteReg (LP1L->b2, I2C_ADDR_ADAU1702W, SAFELOAD_DATA4,SAFELOAD_ADDR4, MOD_CROSSOVER_ALG0_LOW_FILT1_PARAMB2_ADDR); 
		safeWriteReg (LP1L->a2, I2C_ADDR_ADAU1702W, SAFELOAD_DATA5,SAFELOAD_ADDR5, MOD_CROSSOVER_ALG0_LOW_FILT1_PARAMA2_ADDR); 
                set_core_IST_bit(I2C_ADDR_ADAU1702W);
              //LP2L - alg 1 low_filt 1
                safeWriteReg (LP2L->b0, I2C_ADDR_ADAU1702W, SAFELOAD_DATA1,SAFELOAD_ADDR1, MOD_CROSSOVER_ALG1_LOW_FILT1_PARAMB0_ADDR); 
		safeWriteReg (LP2L->b1, I2C_ADDR_ADAU1702W, SAFELOAD_DATA2,SAFELOAD_ADDR2, MOD_CROSSOVER_ALG1_LOW_FILT1_PARAMB1_ADDR); 
		safeWriteReg (LP2L->a1, I2C_ADDR_ADAU1702W, SAFELOAD_DATA3,SAFELOAD_ADDR3, MOD_CROSSOVER_ALG1_LOW_FILT1_PARAMA1_ADDR); 
		safeWriteReg (LP2L->b2, I2C_ADDR_ADAU1702W, SAFELOAD_DATA4,SAFELOAD_ADDR4, MOD_CROSSOVER_ALG1_LOW_FILT1_PARAMB2_ADDR); 
		safeWriteReg (LP2L->a2, I2C_ADDR_ADAU1702W, SAFELOAD_DATA5,SAFELOAD_ADDR5, MOD_CROSSOVER_ALG1_LOW_FILT1_PARAMA2_ADDR); 
                set_core_IST_bit(I2C_ADDR_ADAU1702W);
               //LP1R - alg0 low filt2 
                safeWriteReg (LP1R->b0, I2C_ADDR_ADAU1702W, SAFELOAD_DATA1,SAFELOAD_ADDR1, MOD_CROSSOVER_ALG0_LOW_FILT2_PARAMB0_ADDR); 
		safeWriteReg (LP1R->b1, I2C_ADDR_ADAU1702W, SAFELOAD_DATA2,SAFELOAD_ADDR2, MOD_CROSSOVER_ALG0_LOW_FILT2_PARAMB1_ADDR); 
		safeWriteReg (LP1R->a1, I2C_ADDR_ADAU1702W, SAFELOAD_DATA3,SAFELOAD_ADDR3, MOD_CROSSOVER_ALG0_LOW_FILT2_PARAMA1_ADDR); 
		safeWriteReg (LP1R->b2, I2C_ADDR_ADAU1702W, SAFELOAD_DATA4,SAFELOAD_ADDR4, MOD_CROSSOVER_ALG0_LOW_FILT2_PARAMB2_ADDR); 
		safeWriteReg (LP1R->a2, I2C_ADDR_ADAU1702W, SAFELOAD_DATA5,SAFELOAD_ADDR5, MOD_CROSSOVER_ALG0_LOW_FILT2_PARAMA2_ADDR); 
                set_core_IST_bit(I2C_ADDR_ADAU1702W);
                //LP2R - alg1 low filt2
                safeWriteReg (LP2R->b0, I2C_ADDR_ADAU1702W, SAFELOAD_DATA1,SAFELOAD_ADDR1, MOD_CROSSOVER_ALG1_LOW_FILT2_PARAMB0_ADDR); 
		safeWriteReg (LP2R->b1, I2C_ADDR_ADAU1702W, SAFELOAD_DATA2,SAFELOAD_ADDR2, MOD_CROSSOVER_ALG1_LOW_FILT2_PARAMB1_ADDR); 
		safeWriteReg (LP2R->a1, I2C_ADDR_ADAU1702W, SAFELOAD_DATA3,SAFELOAD_ADDR3, MOD_CROSSOVER_ALG1_LOW_FILT2_PARAMA1_ADDR); 
		safeWriteReg (LP2R->b2, I2C_ADDR_ADAU1702W, SAFELOAD_DATA4,SAFELOAD_ADDR4, MOD_CROSSOVER_ALG1_LOW_FILT2_PARAMB2_ADDR); 
		safeWriteReg (LP2R->a2, I2C_ADDR_ADAU1702W, SAFELOAD_DATA5,SAFELOAD_ADDR5, MOD_CROSSOVER_ALG1_LOW_FILT2_PARAMA2_ADDR); 
                set_core_IST_bit(I2C_ADDR_ADAU1702W);
                //HP1L - alg0 high filt 1
                safeWriteReg (HP1L->b0, I2C_ADDR_ADAU1702W, SAFELOAD_DATA1,SAFELOAD_ADDR1, MOD_CROSSOVER_ALG0_HIGH_FILT1_PARAMB0_ADDR); 
		safeWriteReg (HP1L->b1, I2C_ADDR_ADAU1702W, SAFELOAD_DATA2,SAFELOAD_ADDR2, MOD_CROSSOVER_ALG0_HIGH_FILT1_PARAMB1_ADDR); 
		safeWriteReg (HP1L->a1, I2C_ADDR_ADAU1702W, SAFELOAD_DATA3,SAFELOAD_ADDR3, MOD_CROSSOVER_ALG0_HIGH_FILT1_PARAMA1_ADDR); 
		safeWriteReg (HP1L->b2, I2C_ADDR_ADAU1702W, SAFELOAD_DATA4,SAFELOAD_ADDR4, MOD_CROSSOVER_ALG0_HIGH_FILT1_PARAMB2_ADDR); 
		safeWriteReg (HP1L->a2, I2C_ADDR_ADAU1702W, SAFELOAD_DATA5,SAFELOAD_ADDR5, MOD_CROSSOVER_ALG0_HIGH_FILT1_PARAMA2_ADDR); 
                set_core_IST_bit(I2C_ADDR_ADAU1702W);
                //HP2L - alg 1 high filt1
                safeWriteReg (HP2L->b0, I2C_ADDR_ADAU1702W, SAFELOAD_DATA1,SAFELOAD_ADDR1, MOD_CROSSOVER_ALG1_HIGH_FILT1_PARAMB0_ADDR); 
		safeWriteReg (HP2L->b1, I2C_ADDR_ADAU1702W, SAFELOAD_DATA2,SAFELOAD_ADDR2, MOD_CROSSOVER_ALG1_HIGH_FILT1_PARAMB1_ADDR); 
		safeWriteReg (HP2L->a1, I2C_ADDR_ADAU1702W, SAFELOAD_DATA3,SAFELOAD_ADDR3, MOD_CROSSOVER_ALG1_HIGH_FILT1_PARAMA1_ADDR); 
		safeWriteReg (HP2L->b2, I2C_ADDR_ADAU1702W, SAFELOAD_DATA4,SAFELOAD_ADDR4, MOD_CROSSOVER_ALG1_HIGH_FILT1_PARAMB2_ADDR); 
		safeWriteReg (HP2L->a2, I2C_ADDR_ADAU1702W, SAFELOAD_DATA5,SAFELOAD_ADDR5, MOD_CROSSOVER_ALG1_HIGH_FILT1_PARAMA2_ADDR); 
                set_core_IST_bit(I2C_ADDR_ADAU1702W);
                //HP1R - alg 0 high filt 2
                safeWriteReg (HP1R->b0, I2C_ADDR_ADAU1702W, SAFELOAD_DATA1,SAFELOAD_ADDR1, MOD_CROSSOVER_ALG0_HIGH_FILT2_PARAMB0_ADDR); 
		safeWriteReg (HP1R->b1, I2C_ADDR_ADAU1702W, SAFELOAD_DATA2,SAFELOAD_ADDR2, MOD_CROSSOVER_ALG0_HIGH_FILT2_PARAMB1_ADDR); 
		safeWriteReg (HP1R->a1, I2C_ADDR_ADAU1702W, SAFELOAD_DATA3,SAFELOAD_ADDR3, MOD_CROSSOVER_ALG0_HIGH_FILT2_PARAMA1_ADDR); 
		safeWriteReg (HP1R->b2, I2C_ADDR_ADAU1702W, SAFELOAD_DATA4,SAFELOAD_ADDR4, MOD_CROSSOVER_ALG0_HIGH_FILT2_PARAMB2_ADDR); 
		safeWriteReg (HP1R->a2, I2C_ADDR_ADAU1702W, SAFELOAD_DATA5,SAFELOAD_ADDR5, MOD_CROSSOVER_ALG0_HIGH_FILT2_PARAMA2_ADDR); 
                set_core_IST_bit(I2C_ADDR_ADAU1702W);
                //HP2R - alg 1 high filt 2
                safeWriteReg (HP2R->b0, I2C_ADDR_ADAU1702W, SAFELOAD_DATA1,SAFELOAD_ADDR1, MOD_CROSSOVER_ALG1_HIGH_FILT2_PARAMB0_ADDR); 
		safeWriteReg (HP2R->b1, I2C_ADDR_ADAU1702W, SAFELOAD_DATA2,SAFELOAD_ADDR2, MOD_CROSSOVER_ALG1_HIGH_FILT2_PARAMB1_ADDR); 
		safeWriteReg (HP2R->a1, I2C_ADDR_ADAU1702W, SAFELOAD_DATA3,SAFELOAD_ADDR3, MOD_CROSSOVER_ALG1_HIGH_FILT2_PARAMA1_ADDR); 
		safeWriteReg (HP2R->b2, I2C_ADDR_ADAU1702W, SAFELOAD_DATA4,SAFELOAD_ADDR4, MOD_CROSSOVER_ALG1_HIGH_FILT2_PARAMB2_ADDR); 
		safeWriteReg (HP2R->a2, I2C_ADDR_ADAU1702W, SAFELOAD_DATA5,SAFELOAD_ADDR5, MOD_CROSSOVER_ALG1_HIGH_FILT2_PARAMA2_ADDR); 
                set_core_IST_bit(I2C_ADDR_ADAU1702W);
                delay(50);
}

//Next 3 functions write the boot code to the DSP and initialize sound 
//Writes program memory in 5 byte chunks
void SIGMA_WRITE_REGISTER_PROG(int devAddress, int address, int length_reg, ADI_REG_TYPE* pData){
    int addtemp = 0;
    int addcount =0, regcount=0;
       addtemp = address;
       #ifndef cbi
       #define cbi(sfr, bit) (_SFR_BYTE(sfr) &= ~_BV(bit))
       #endif
       for(regcount=0;regcount<length_reg;regcount++)
       {
                Wire.beginTransmission(devAddress);
                Wire.write(((addtemp +regcount) & 0xFF00) >> 8);
                Wire.write((addtemp+regcount) & 0x00FF);
                
                Wire.write(pgm_read_byte(pData+addcount));
                Wire.write(pgm_read_byte(pData+addcount+1));
                Wire.write(pgm_read_byte(pData+addcount+2));
                Wire.write(pgm_read_byte(pData+addcount+3));
                Wire.write(pgm_read_byte(pData+addcount+4));
                addcount+=5;
                Wire.endTransmission();
       }
 }
//Writes parameter memory in 4 byte chunks 
void SIGMA_WRITE_REGISTER_PARAM(int devAddress, int address, int length_reg, ADI_REG_TYPE* pData){
int addtemp = 0;
int addcount =0, regcount=0;
   addtemp = address;
   #ifndef cbi
   #define cbi(sfr, bit) (_SFR_BYTE(sfr) &= ~_BV(bit))
   #endif
//   Serial.begin(9600);
   for(regcount=0;regcount<length_reg;regcount++)
   {
            Wire.beginTransmission(devAddress);
            Wire.write(((addtemp +regcount) & 0xFF00) >> 8);
            Wire.write((addtemp+regcount) & 0x00FF);
            Wire.write(pgm_read_byte(pData+addcount));
           
            Serial.print(pgm_read_byte(pData+addcount),HEX);
            Serial.print(", ");
            Wire.write(pgm_read_byte(pData+addcount+1));
            Serial.print(pgm_read_byte(pData+addcount+1),HEX);
            Serial.print(", ");
            Wire.write(pgm_read_byte(pData+addcount+2));
            Serial.print(pgm_read_byte(pData+addcount+2),HEX);
            Serial.print(", ");
            Wire.write(pgm_read_byte(pData+addcount+3));
            Serial.print(pgm_read_byte(pData+addcount+3),HEX);
            Serial.println(", ");
            addcount+=4;
            Wire.endTransmission();
   }
//   Serial.end();
 }
//Write small continuous blocks of data to DSP, such as the core register and other hardware parameters
void SIGMA_WRITE_REGISTER_BLOCK(int devAddress, int address, int length, ADI_REG_TYPE* pData){
   #ifndef cbi
#define cbi(sfr, bit) (_SFR_BYTE(sfr) &= ~_BV(bit))
#endif
int i=0;

        Wire.beginTransmission(devAddress);
          
          Wire.write(((address) & 0xFF00) >> 8);
          Wire.write((address) & 0x00FF);
 //         Serial.println(((address) & 0xFF00) >> 8,HEX);
 //         Serial.println((address) & 0x00FF,HEX);
           
           
          for (i>0;i<length;i++){
//           Serial.println(pgm_read_byte(pData+i),HEX);
           Wire.write(pgm_read_byte(pData+i));
         }
         Wire.endTransmission();
}

/* 
 * Convert a floating-point value to SigmaDSP (5.23) fixed point format 
 */
#define SIGMASTUDIOTYPE_FIXPOINT_CONVERT( _value )(_value * 0x00800000)

//Writes one filter based on 5 passed parameter values and a starting destination address
void safeLoadFilter( Biquad* filterX, int addressStart)
{
//Load 5 filter coefficients into DSP safe-load registers, note a0 corresponds to b0 in SigmaDSP due to a naming convention conflict
// Function call format: safeLoadFilter(filter1, address);
        
        safeWriteReg (filterX->b0, I2C_ADDR_ADAU1702W, SAFELOAD_DATA1,SAFELOAD_ADDR1,  addressStart); 
	safeWriteReg (filterX->b1, I2C_ADDR_ADAU1702W, SAFELOAD_DATA2,SAFELOAD_ADDR2, (addressStart+1)); 
	safeWriteReg (filterX->b2, I2C_ADDR_ADAU1702W, SAFELOAD_DATA3,SAFELOAD_ADDR3, (addressStart+2)); 
	safeWriteReg (filterX->a1, I2C_ADDR_ADAU1702W, SAFELOAD_DATA4,SAFELOAD_ADDR4, (addressStart+3)); 
	safeWriteReg (filterX->a2, I2C_ADDR_ADAU1702W, SAFELOAD_DATA5,SAFELOAD_ADDR5, (addressStart+4)); 
//Send the OK to transmit filter
        set_core_IST_bit(I2C_ADDR_ADAU1702W);	
}

//Writes 1 parameter register via Safeload Write over I2C, doesn't include IST call 
void safeWriteReg(double coeff,short deviceaddress, short data_reg, short addr_reg, short address)
{
   long fixedPtCoeff = 0;
   unsigned char bytes[4]={0}; // 4 x Chars to "carry" the broken Float value

   fixedPtCoeff = SIGMASTUDIOTYPE_FIXPOINT_CONVERT(coeff) ; //Convert to 5.23 fixed point format
// Seperate fixed point variable into 4 byte sized chunks for transmission to the DSP  
   bytes[0] = (fixedPtCoeff >> 24) & 0xFF;
   bytes[1] = (fixedPtCoeff >> 16) & 0xFF;
   bytes[2] = (fixedPtCoeff >> 8) & 0xFF;
   bytes[3] = fixedPtCoeff & 0xFF;   
   
   //Transmit Data//
   Wire.beginTransmission(deviceaddress);  // 0x34 (write port)
   //send data register address 1 byte at a time
   Wire.write(((data_reg >> 8) & 0xFF)); // send upper byte safeload data register
   Wire.write((data_reg & 0xFF));        // send lower byte 
   Wire.write(0x00); //load 00 into byte 3 of safeload register as required by safeload write format
   Wire.write(bytes,4); // write 4 bytes of parameter data
   Wire.endTransmission();      
   
   Wire.beginTransmission(deviceaddress); // 0x34 (write port)
   //send address register 1 byte at a time
   Wire.write((addr_reg >> 8) & 0xFF); // send upper byte (0x08), safeload address register 
   Wire.write(addr_reg & 0xFF);        // send lower byte         
   Wire.write((address >> 8) & 0xFF); // destination register
   Wire.write(address & 0xFF);              
   
   Wire.endTransmission();
}


void subModeEnable(int subModeToggleAdd, int rightMuteAdd, int subModeGainAdd){
    //Enable submode toggle by setting mute to "1" -> pass audio
    safeWriteReg(UNO,I2C_ADDR_ADAU1702W,  SAFELOAD_DATA1,  SAFELOAD_ADDR1,  subModeToggleAdd);
    //Enable rightMute 
    safeWriteReg(ZERO,I2C_ADDR_ADAU1702W,  SAFELOAD_DATA2,  SAFELOAD_ADDR2, rightMuteAdd);
    // Balance sub with mains by allowing half of left and half of right into the bass mix
    safeWriteReg(HALF_POWER,I2C_ADDR_ADAU1702W,  SAFELOAD_DATA3,  SAFELOAD_ADDR3, subModeGainAdd);
    // Transfer the settings
    set_core_IST_bit(I2C_ADDR_ADAU1702W);
}


void set_core_IST_bit(int deviceaddress)
  {   
    /*set the initiate safeload transfer bit ( 5th bit )
    in the core control register to initiate the loading into RAM*/
    Wire.beginTransmission(deviceaddress);  
    Wire.write(0x08);
    Wire.write(0x1C);  //address of DSP core control: 0x081C. Holds 2bytes data.
    Wire.write(0x00);
    Wire.write(0x3D); // 96 kHz, IST high
    Wire.endTransmission();
  }
int importFilter(Biquad* filter1, Biquad* filter2,Biquad* filter3,Biquad* filter4,Biquad* filter5,Biquad* filter6, Biquad* filter7,Biquad* filter8,Biquad* filter9, Biquad* filter10){

  
    const int MAX_CHARS_PER_LINE = 100;
    const int MAX_TOKENS_PER_LINE = 20;
    const char* const DELIMITER = " ";
    int filterIndex = 1;
    double fcTemp = 0;
    double gainTemp = 0;
    double qTemp = 0;
    int address = 0, n = 0, i = 0;
    char charTemp;
    int type;
    int subModeFlag = 0, memlength = 0;
/*  
      // If Had file system
      /*FILE *fr; // Declare file pointer
      // create a file-reading object
      fr = fopen("test.txt","r"); // open a file
*/  
// Load first character into temp variable (need for != Null condition)
     charTemp = EEPROM.read(address);
     address++;
      while (charTemp != NULL){ // stop at end of EEPROM
      
        char buf[MAX_CHARS_PER_LINE];
        const char* token[MAX_TOKENS_PER_LINE] = {}; // initialize to 0
        while (charTemp != '\n'){
          // Throw out commas
            if (charTemp!= ','){
            buf[i] = charTemp;
            }
            else{
              i--; //Still looking for a valid character (counteracts i++)
            }
            charTemp = EEPROM.read(address); 
            address++;
            i++;
          }
          buf[i] = '\0';
        //  Serial.println(buf);
        // Reset buf index
        i = 0;
        // parse the line // first token
        token[0] = strtok(buf, DELIMITER);
        //Serial.println(token[0]);
        if (token[0]) // zero if line is blank
        {
          for (n = 1; n < MAX_TOKENS_PER_LINE; n++)
          {
            token[n] = strtok(0, DELIMITER); // subsequent tokens
            if (!token[n])
              break; // no more tokens
            else;
//              Serial.println(token[n]); // Print each parsed word
          }
        }
        // Process a given line -> set filter
        // If declared submode, rest of line will be blank, set flag and go to next line//
             if (!strcmp(token[0], "SUBMODE")){
               subModeFlag = 1;   
             }
        // This is used to ignore header lines that define REW parameters//
             else if (!strcmp(token[0], "Filter")){ // filter may occur
        // If filter is disabled, don't implement it        
                if(!strcmp(token[2], "ON")){ //filter is enabled
        // Compare substring to find what type of filter is 
        // contained in the 4th word (see format)
        
        // Note: all token[x] refer to the word # in the line - 1 (index starts at 0)
        
                   if(!strncmp(token[3],"B",1)){ // Peak filter

                     fcTemp = atof(token[5]);
                     gainTemp = atof(token[8]);
                     qTemp = atof(token[11]);
                     type = bq_type_peak;
                   }
                   else if(!strncmp(token[3],"LS",2)){ // lowshelf
                      qTemp = 1;
                      fcTemp = atof(token[6]);
                      gainTemp = atof(token[9]);
                      type = bq_type_lowshelf;
                   }
                   else if(!strncmp(token[3],"HS",2)){ // highshelf
                      qTemp = 1;
                      fcTemp = atof(token[6]);
                      gainTemp = atof(token[9]);
                      type = bq_type_highshelf;
                   }
                   else if(!strncmp(token[3],"HP",2)){ // highpass 2nd order butterworth
                      qTemp = 1;
                      gainTemp = 0;
                      fcTemp = atof(token[5]); // change to index 6 if include 12dB/24dB designation in input file
                      type = bq_type_highpassLR;
                   }
                   else if(!strncmp(token[3],"LP",2)){ // lowpass 2nd order butterworth stage
                      qTemp = 1;
                      gainTemp = 0;
                      fcTemp = atof(token[5]); // change to index 6 if include 12dB/24dB designation in input file
                      type = bq_type_lowpassLR; 

                   }
                   else if(!strncmp(token[3],"NOTCH",5)){ // notch filter
                      qTemp = atof(token[8]); // Slope parameter, 0 to 2
                      gainTemp = 0;             
                      fcTemp = atof(token[5]); // 
                      type = bq_type_notch; 

                   }
                   else if(!strncmp(token[3],"NONE",4)||!strncmp(token[3],"Modal",4)){
                        gainTemp = 0;
                        qTemp = 0.707;
                        type = bq_type_lowshelf;
                   }
              // Note: Modal filter not supported inserted values are default "do nothing" filter
              // Bipass filters 9-10 for memory conservation
                        
                        if (filterIndex == 7){
                          filterIndex = 999; // out of range
                        }
                       
                       switch(filterIndex){
                         case 1:
                         
                         filter1 ->setBiquad(type, fcTemp / sampleRate, qTemp, gainTemp);
/*                         Serial.begin(9600);
                         Serial.print("type is: ");
                         Serial.println(filter1->type,8);
                         Serial.print("fc is: ");
                         Serial.println(filter1->Fc*96000,8);
                         Serial.print("gain is: ");
                         Serial.println(filter1->peakGain,8);
                         Serial.print("Q is: ");
                         Serial.println(filter1->Q,8);
                         Serial.println(filter1->b0,8);
                         Serial.println(filter1->b1,8);
                         Serial.println(filter1->b2,8);
                         Serial.println(filter1->a1,8);
                         Serial.println(filter1->a2,8);
*/
                         break;
                         
                         case 2:
                         filter2 ->setBiquad(type, fcTemp / sampleRate, qTemp, gainTemp);               
                         break;
                         
                         case 3:
                         filter3 ->setBiquad(type, fcTemp / sampleRate, qTemp, gainTemp);
                         break;
                         
                         case 4:
                         filter4 ->setBiquad(type, fcTemp / sampleRate, qTemp, gainTemp);
                         break;
                         
                         case 5:
                         filter5 ->setBiquad(type, fcTemp / sampleRate, qTemp, gainTemp);
                         break;

                         case 6:
                         filter6 ->setBiquad(type, fcTemp / sampleRate, qTemp, gainTemp);
                         break;
/*
                         case 7:
                         filter7 ->setBiquad(type, fcTemp / sampleRate, qTemp, gainTemp);
                         break;
                         
                         case 8:
                         filter8 ->setBiquad(type, fcTemp / sampleRate, qTemp, gainTemp);
                         break;
                         
                         case 9:
                         filter9 ->setBiquad(type, fcTemp / sampleRate, qTemp, gainTemp);
                         break;
                         
                         case 10:
                         filter10 ->setBiquad(type, fcTemp / sampleRate, qTemp, gainTemp);
                         break;
*/                         
                         default:
                         break; //Do nothing
                }//End case switch
                filterIndex++; // filter implemented, go to next open filter
             }//Ends 'on if'         
        
      }//Ends 'filter if'
        charTemp = EEPROM.read(address); // Read out start of next line
        address++;
      }//Done with reading

      if (subModeFlag){
      return SUB_MODE_FLAG_HIGH; // evaluates to 2
      }
      else
      return 0;          
}
#endif
